@@ -4,18 +4,18 @@ use warnings;
 use strict;
 use 5.010001;
 
-our $VERSION = '0.021_02';
+our $VERSION = '0.019';
 use Exporter 'import';
 our @EXPORT_OK = qw( choose );
 
 use Parse::ANSIColor::Tiny qw();
 use Term::ANSIColor        qw( colored );
 use Text::ANSI::WideUtil   qw( ta_mbtrunc );
+use Unicode::GCString      qw();
 
 use if $^O eq 'MSWin32', 'Win32::Console::ANSI';
 
 use Term::Choose::Constants qw( :choose :linux );
-use Term::Choose::LineFold qw( print_columns );
 
 use parent 'Term::Choose';
 
@@ -49,24 +49,30 @@ sub choose {
 
 sub __copy_orig_list {
     my ( $self ) = @_;
-    $self->{list} = [ @{$self->{orig_list}} ];
     if ( $self->{ll} ) {
-        for ( @{$self->{list}} ) {
-            $_ = $self->{undef} if ! defined $_;
-        }
+        $self->{list} = [ map {
+            my $copy = $_;
+            if ( ! $copy ) {
+                $copy = $self->{undef} if ! defined $copy;
+                $copy = $self->{empty} if $copy eq '';
+            }
+            $copy;
+        } @{$self->{orig_list}} ];
     }
     else {
-        for ( @{$self->{list}} ) {
-            if ( ! $_ ) {
-                $_ = $self->{undef} if ! defined $_;
-                $_ = $self->{empty} if $_ eq '';
+        $self->{list} = [ map {
+            my $copy = $_;
+            if ( ! $copy ) {
+                $copy = $self->{undef} if ! defined $copy;
+                $copy = $self->{empty} if $copy eq '';
             }
-            if ( ref ) {
-                $_ = sprintf "%s(0x%x)", ref $_, $_;
+            if ( ref $copy ) {
+                $copy = sprintf "%s(0x%x)", ref $copy, $copy;
             }
-            s/\p{Space}/ /g;               # replace, but don't squash sequences of spaces
-            s/\p{C}/$&=~m|\e| && $&/eg;    # remove \p{C} but keep \e
-        }
+            $copy =~ s/\p{Space}/ /g;               # replace, but don't squash sequences of spaces
+            $copy =~ s/\p{C}/$&=~m|\e| && $&/eg;    # remove \p{C} but keep \e
+            $copy;
+        } @{$self->{orig_list}} ];
     }
 }
 
@@ -76,14 +82,15 @@ sub __unicode_trim {
     return ta_mbtrunc( $str, $len );
 }
 
-sub _strip_ansi_color {
-    ( my $str = $_[0] ) =~ s/\e\[[\d;]*m//msg;
-    return $str;
-}
 
 sub __print_columns {
     #my $self = $_[0];
-    print_columns( _strip_ansi_color( $_[1] ) );
+    Unicode::GCString->new( __strip_ansi_color( $_[1] ) )->columns();
+}
+
+sub __strip_ansi_color {
+    ( my $str = $_[0] ) =~ s/\e\[[\d;]*m//msg;
+    return $str;
 }
 
 
@@ -185,7 +192,7 @@ Term::Choose_HAE - Choose items from a list interactively.
 
 =head1 VERSION
 
-Version 0.021_02
+Version 0.019
 
 =cut
 
@@ -264,21 +271,13 @@ C<Term::Choose_HAE> inherits the options from L<Term::Choose|Term::Choose/OPTION
 If I<fill_up> is enabled, the highlighting of the cursor position and in list context the highlighting of the selected
 items has always the width of the column.
 
-=over
-
-=item
-
-I<fill_up> set to C<1>: the color of the highlighting of leading and trailings spaces is set to the color of
+If I<fill_up> is set to C<1>, the color of the highlighting of leading and trailings spaces is set to the color of
 the highlighting of the adjacent non-space character of the item if these spaces are not embedded in escape sequences.
 
-=item
-
-I<fill_up> set to C<2>: leading and trailings spaces are highlighted with the default color for highlighting if
+If I<fill_up> is set to C<2>, leading and trailings spaces are highlighted with the default color for highlighting if
 these spaces are not embedded in escape sequences.
 
-=back
-
-If I<fill_up> is disabled, leading and trailing spaces are not highlighted if they are not embedded in escape sequences.
+If I<fill_up> is disabled, leading and trailing spaces are not highlighted if the are not embedded in escape sequences.
 
 =head1 REQUIREMENTS
 
@@ -308,7 +307,7 @@ L<stackoverflow|http://stackoverflow.com> for the help.
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (C) 2015-2016 Matthäus Kiem.
+Copyright (C) 2015 Matthäus Kiem.
 
 This library is free software; you can redistribute it and/or modify it under the same terms as Perl 5.10.0. For
 details, see the full text of the licenses in the file LICENSE.
